@@ -20,7 +20,7 @@
 """Conftest helper pytest file: fixtures container here are
  reachable by all tests"""
 import pytest
-import pkg_resources
+import importlib.resources
 import subprocess
 from typing import Mapping, Union, Tuple, List
 from qubesadmin.tests import QubesTest
@@ -424,52 +424,53 @@ def test_qapp_broken():  # pylint: disable=redefined-outer-name
     return qapp
 
 
-SIGNALS_REGISTERED = False
-
 @pytest.fixture
 def test_builder():
     """Test gtk_builder with loaded test glade file and registered signals."""
-    global SIGNALS_REGISTERED  # pylint:disable=global-statement
-    # register all the signals various widgets might emit
-    if not SIGNALS_REGISTERED:
+    try:
         GlobalConfig.register_signals()
-        SIGNALS_REGISTERED = True
+    except RuntimeError:
+        # signals already registered
+        pass
     # test glade file contains very simple setup with correctly named widgets
     builder = Gtk.Builder()
-    builder.add_from_file(pkg_resources.resource_filename(
-        __name__, 'test.glade'))
+    glade_ref = (importlib.resources.files('qubes_config') /
+                 'tests/test.glade')
+    with importlib.resources.as_file(glade_ref) as path:
+        builder.add_from_file(str(path))
     return builder
 
 @pytest.fixture
 def real_builder():
     """Gtk builder with actual config glade file registered"""
-    global SIGNALS_REGISTERED  # pylint:disable=global-statement
-    # register all the signals various widgets might emit
-    if not SIGNALS_REGISTERED:
+    try:
         GlobalConfig.register_signals()
-        SIGNALS_REGISTERED = True
+    except RuntimeError:
+        # signals already registered
+        pass
     # test glade file contains very simple setup with correctly named widgets
     builder = Gtk.Builder()
-    builder.add_from_file(pkg_resources.resource_filename(
-        'qubes_config', 'global_config.glade'))
+    glade_ref = (importlib.resources.files('qubes_config') /
+                 'global_config.glade')
+    with importlib.resources.as_file(glade_ref) as path:
+        builder.add_from_file(str(path))
     return builder
 
 
-NEW_QUBE_SIGNALS_REGISTERED = False
-
-
 @pytest.fixture
 def new_qube_builder():
     """Gtk builder with actual config glade file registered"""
-    global NEW_QUBE_SIGNALS_REGISTERED  # pylint:disable=global-statement
-    # register all the signals various widgets might emit
-    if not NEW_QUBE_SIGNALS_REGISTERED:
+    try:
         CreateNewQube.register_signals()
-        NEW_QUBE_SIGNALS_REGISTERED = True
+    except RuntimeError:
+        # signals already registered
+        pass
     # test glade file contains very simple setup with correctly named widgets
     builder = Gtk.Builder()
-    builder.add_from_file(pkg_resources.resource_filename(
-        'qubes_config', 'new_qube.glade'))
+    glade_ref = (importlib.resources.files('qubes_config') /
+                 'new_qube.glade')
+    with importlib.resources.as_file(glade_ref) as path:
+        builder.add_from_file(str(path))
     return builder
 
 
