@@ -27,7 +27,7 @@ from html import escape
 
 import gettext
 
-import pkg_resources
+import importlib.resources
 from datetime import datetime
 
 t = gettext.translation("desktop-linux-manager", fallback=True)
@@ -37,7 +37,8 @@ import gi  # isort:skip
 gi.require_version('Gtk', '3.0')  # isort:skip
 from gi.repository import Gtk  # isort:skip
 
-EOL_DATES = json.load(pkg_resources.resource_stream(__name__, 'eol.json'))
+with importlib.resources.files('qui').joinpath('eol.json').open() as stream:
+    EOL_DATES = json.load(stream)
 # remove the following suffixes when checking for EOL
 SUFFIXES = ['-minimal', '-xfce']
 
@@ -83,13 +84,17 @@ def run_asyncio_and_show_errors(loop, tasks, name, restart=True):
 def check_support(vm):
     """Return true if the given template/standalone vm is still supported, by
     default returns true"""
-    template_name: str = vm.features.get('template-name', '')
-    if not template_name:
-        return True
-    for suffix in SUFFIXES:
-        template_name = template_name.removesuffix(suffix)
-    eol = EOL_DATES.get(template_name, None)
-    if not eol:
-        return True
-    eol = datetime.strptime(eol, '%Y-%m-%d')
+    # first, check if qube itself has known eol
+    eol_string: str = vm.features.get('os-eol', '')
+
+    if not eol_string:
+        template_name: str = vm.features.get('template-name', '')
+        if not template_name:
+            return True
+        for suffix in SUFFIXES:
+            template_name = template_name.removesuffix(suffix)
+        eol_string = EOL_DATES.get(template_name, None)
+        if not eol_string:
+            return True
+    eol = datetime.strptime(eol_string, '%Y-%m-%d')
     return eol > datetime.now()
